--
-- Project:  LuaIDL
-- Version:  0.5.1b
-- Author:   Ricardo Calheiros <rcosme@tecgraf.puc-rio.br>
-- Last modification: 26/09/05 14:15
-- Filename: pre.lua
-- 

local io     = require "io"
local os     = require "os"
local pairs  = pairs
local string = require "string"
local table  = require "table"

module( 'luaidl.pre' )

--
--CPP SUPPORT BEGIN
CPP_PATH   = 'cpp '
CPP_PARAMS = '-o %s '

function idlpp( idlfilename, prefilename )
  return os.execute( CPP_PATH..string.format( CPP_PARAMS, prefilename )..idlfilename )
end
--CPP SUPPORT END
--

local tab_macros = { }
local currNumLine = 1
local currFilename
local READ = true
local homedir

local function newStack ()
  return {""}
end

local function addString (stack, s)
  table.insert(stack, s)
  for i=table.getn(stack)-1, 1, -1 do
    if string.len(stack[i]) > string.len(stack[i+1]) then
      break
    end
    stack[i] = stack[i] .. table.remove(stack)
  end
end

local function processDirective(...)
  local directive = arg[ 1 ]
  if ( directive == 'endif' ) then
    READ = true
    return ''
  end --if
  if ( READ ) then 
    if ( directive == 'define' ) then
      local macro = arg[ 2 ]
      local value = arg[ 3 ]
      tab_macros[ macro ]  = value
    elseif ( directive == 'include' ) then
      local incFilename = string.sub( arg[ 2 ], 2, -2 )
      fh, e = io.open( homedir..incFilename )
      local incSource = fh:read( '*a' )
      local incENDNumLine = currNumLine + 1
      local OUTFilename = currFilename
      incSource = _run( incSource, incFilename )
      return string.format( '# %d "%s" 1\n%s# %d "%s" 2\n', 
                            1, incFilename, 
                            incSource,
                            incENDNumLine, OUTFilename
                          )
    elseif ( directive == 'ifndef' ) then
      local macro = arg[ 2 ]
      if ( tab_macros[ macro ] ) then
        READ = false
      end --if
    else
      return '#'..table.concat(arg, ' ')
    end --if
  end --if
  return ''
end

local function macroExpansion( str )
  for name, value in pairs( tab_macros ) do
    str = string.gsub( str, '([^%w])'..name..'([^%w])', '%1'..value..'%2' )
  end --for
  return str
end

function run( source, pfilename )
  local r = _run( source, pfilename )
  tab_macros = { }
  currNumLine = 1
  currFilename = nil
  READ = true
  homedir = nil
  return r
end

function _run( source, pfilename )
  if not pfilename then
    pfilename = ''
  end --if
  local num  
  local output = newStack()
  if ( not currFilename ) then
    addString( output, string.format( '# 1 "%s"\n', pfilename ) )
    homedir, num = string.gsub( pfilename, '(.*/).*', '%1' )   
    if ( num == 0 ) then
      homedir = ''
    end --if
  end --if
  currFilename = pfilename
  local numLine = 1
  for strLine in string.gfind( source, "(.-\n)" ) do
    strLine = string.gsub( strLine, '^%s*#%s*(%w+)%s*([%w-_$."><]*)%s*(%w*)', processDirective )
    if ( READ ) then
      strLine = macroExpansion( strLine )
      addString( output, strLine )
    end --if
    numLine = numLine + 1
    currNumLine = numLine
    currFilename = pfilename
  end --for
  return table.concat( output )
end
