--
-- Project:  LuaIDL
-- Version:  0.3.7b
-- Author:   Ricardo Calheiros <rcosme@tecgraf.puc-rio.br>
-- Last modification: 13/07/05
-- Filename: init.lua
-- 

local luaidl        = require "luaidl"
local serialization = require "serialization"
local viewer        = require "loop.debug.Viewer"

viewer = viewer{
	identation = "|  ",
	maxdepth = 1,
}

local function pack(...) return arg end
local function message(level, message, ...)
	local prefix = string.rep(viewer.identation, level)
	for index, value in ipairs(arg) do
		arg[index] = viewer:tostring(value, prefix)
	end
	print "------------------------------------------------------------"
	io.stderr:write( prefix, string.format(message, unpack(arg)), "\n")
end
local function equals(value, other, history, level)
	level = level and level + 1 or 1
	if not history then history = {} end
	if value == other then
		return true
	elseif type(value) == type(other) then
		if history[value] == other then
			return true
		elseif not history[value] and type(value) == "table" then
			history[value] = other
			local keysfound = {}
			for key, field in pairs(value) do
				local otherfield = other[key]
				if otherfield == nil then
					local success = false
					for otherkey, otherfield in pairs(other) do
						if
							equals(key, otherkey, history, level) and
							equals(field, otherfield, history, level)
						then
							keysfound[otherkey] = true
							success = true
							break
						end
					end
					if not success then
						message(level, "unable to match new field %s on old table: %s",
						        key, other)
						return false
					end
				elseif equals(field, otherfield, history, level) then
					keysfound[key] = true
				else
					message(level, "value of field %s is now: %s, but was: %s",
					        key, field, otherfield)
					return false
				end
			end
			for otherkey, otherfield in pairs(other) do
				if not keysfound[otherkey] then
					message(level, "field %s vanished, but used to be: %s",
					        otherkey, otherfield)
					return false
				end
			end
			return true
		end
	end
	return false
end

local database = assert(io.open("luaidl/db/index.lua"))
database = database:read("*a")
database = assert(loadstring("return {"..database.."}", "IDL database index"))()

for _, entry in ipairs( database ) do
	local idlfile = "luaidl/db/"..entry
	local luafile = string.gsub(idlfile, ".idl$", ".lua")
	local old = nil
	local new = pack(assert(luaidl.parsefile(idlfile)))
	local file = io.open(luafile)
	if file then
		old = serialization.load(file:read("*a"))
		file:close()
	end
	print("evaluating new output of file '"..entry.."' ...")
	if not equals(new, old) then
		print("save new output of file '"..entry.."'?")
		local answer = io.read()
		if string.find(answer, "^y") then
			new = serialization.serialize(new)
			file = assert(io.open(luafile, "w"))
			file:write(new)
			file:close()
		end
	end
end
