local error    = error
local require  = require
local rawequal = rawequal
local tostring = tostring

local table = require "table"
local debug = require "debug"
local loop  = require "loop"
local oo    = require "loop.cached"

module("luaunit.framework.Assert",
	loop.define(oo.class({}, require "luaunit.framework.Constructable")) )

local FailureException = require "luaunit.framework.FailureException"
	
ASSERT_EXCEPTION_NAME = "ASSERT EXCEPTION"
FAILURE_MESSAGE = "Assert Failure"
		
function assertEquals(self, expected, actual, message)
	if not (expected == actual) then
		self:fail(table.concat {
			message or "",
			" expected:<",
			tostring(expected),
			"> but was:<",
			tostring(actual),
			">"
		})
	end
end
		
function assertFalse(self, condition, message)
	self:assertTrue(not condition, message);
end
		
function assertNotNil(self, reference, message)
	self:assertTrue(reference ~= nil, message);
end
		
function assertNotSame(self, expected, actual, message)
	if rawequal(expected, actual) then
		self:fail(table.concat{
			message or "",
			" expected not same"
		})
	end
end

function assertNil(self, reference, message)
	self:assertTrue(reference == nil, message);
end

function assertSame(self, expected, actual, message)
	if not rawequal(expected, actual)then
		self:fail(table.concat{
			message or "",
			" expected same:<",
			tostring(expected),
			"> was not:<",
			tostring(actual),
			">"
		})
	end
end

function assertTrue(self, condition, message)
	if not condition then
		self:fail(message)
	end
end

function fail(self, message)
	error(FailureException {
		message = message or self.FAILURE_MESSAGE,
		traceback = debug.traceback(),
	})
end
