local type    = type
local error   = error
local pcall   = pcall
local unpack  = unpack
local require = require

local loop = require "loop"
local oo   = require "loop.scoped"

module("luaunit.framework.TestCase",
	loop.define(oo.class({}, require "luaunit.framework.Assert")),
	loop.seeapi(oo) )

local TestResult = require "luaunit.framework.TestResult"

protected = {}

function constructor(self, name)
	self.name = name
end

function protected:createResult()
	return TestResult()
end

function protected:try_function(func, ...)
	return pcall(function() func(unpack(arg)) end)
end

function public:countTestCases()
	return 1
end

function public:run(result)
	if not result then result = self:createResult() end
	result:run(this(self))
	return result
end

function public:runBare()
	local success, message = true, nil
	if self.setUp then
		success, message = self:try_function(self.setUp, self)
	end
	if success then
		success, message = self:try_function(self.runTest, self)
	end
	if self.tearDown then
		local ended, errmsg = self:try_function(self.tearDown, self)
		if success then
			success, message = ended, errmsg
		elseif not ended then
			message = message .. "\n" .. errmsg
		end
	end
	if not success then
		error(message)
	end
end

function public:runTest()
	self:assertNotNil(self.name, "TestCase has no name")
	local runMethod = this(self)[self.name]
	self:assertNotNil(runMethod, "There's no public member by the name '" .. self.name .. "'")
	self:assertSame("function", type(runMethod), "There's no public method by the name '" .. self.name .. "'")
	runMethod(self)
end
