local type         = type
local error        = error
local pairs        = pairs
local ipairs       = ipairs
local rawset       = rawset
local require      = require
local tostring     = tostring
local getmetatable = getmetatable

local string = require "string"
local table  = require "table"
local loop   = require "loop"
local oo     = require "loop.scoped"

module("luaunit.framework.TestSuite",
	loop.define(oo.class({}, require "luaunit.framework.Constructable")),
	loop.seeapi(oo) )

local TestCase = require "luaunit.framework.TestCase"

private = {}

function constructor(self, class, name)
	rawset(private(self), "fTests", {})

	if type(class) == "string" then
		name = class
		class = nil
	end

	self.name = name
	if class then
		-- Creates a test for each public test??? method in the class and adds it to the test suite
		--for field, value in members(class, { public = true, inherited = true }) do
		for field, value in pairs(getmetatable(class).class) do
			if string.find(field, "test") == 1 and type(value) == "function" then
				self:addTest(class(field))
			end
		end
		if table.getn(self.fTests) == 0 then
			local test = TestCase("warning")
			function test:runTest()
				self:fail("No tests found in " .. tostring(class))
			end
			self:addTest(test)
		end
	end
end
	
function public:addTest(test)
	if test
		then table.insert(self.fTests, test)
		else error("attempt to add 'test' (a nil value).")
	end
end

function public:addTestSuite(class)
	if class
		then self:addTest(_CLASS(class))
		else error("attempt to add 'class' (a nil value).")
	end
end

function public:countTestCases()
	local count = 0
	for _, test in ipairs(self.fTests) do
		count = count + test:countTestCases()
	end
	return count
end

function public:run(result)
	for _, test in ipairs(self.fTests) do
		if result:shouldStop() then break end
		self:runTest(test, result)
	end
end

function public:runTest(test, result)
	test:run(result)
end

function public:testAt(index)
	return self.fTests[index]
end

function public:testCount()
	return table.getn(self.fTests)
end

function public:tests()
	local tests = {}
	for _, test in ipairs(self.fTests) do
		table.insert(tests, test)
	end
	return tests
end
