local rawset  = rawset
local require = require

local io   = require "io"
local os   = require "os"
local loop = require "loop"
local oo   = require "loop.scoped"

module("luaunit.runner.BasicRunner",
	loop.define(oo.class({}, require "luaunit.framework.Constructable")),
	loop.seeapi(oo) )

local TestResult    = require "luaunit.framework.TestResult"
local ResultPrinter = require "luaunit.runner.ResultPrinter"

public.wait = false
protected = {}
	
function constructor(self)
	rawset(protected(self), "fPrinter", ResultPrinter())
end

function protected:pause(wait)
	if not wait then return end
	self.fPrinter:printWaitPrompt();
	io.read();
end

function public:doRun(test, wait)
	local result = TestResult();
	result:addListener(self.fPrinter);
	local startTime = os.time();
	test:run(result);
	local endTime = os.time();
	local runTime = os.difftime(endTime, startTime);
	self.fPrinter:printResult(result, runTime);

	self:pause(wait);
	return result;
end
