local type     = type
local tostring = tostring

local io    = require "io"
local table = require "table"
local loop  = require "loop"
local oo    = require "loop.scoped"

module("luaunit.runner.ResultPrinter",
	loop.define(oo.class({}, require "luaunit.framework.Constructable")),
	loop.seeapi(oo) )

protected = { fColumn = 0 }

function constructor(self)
	self.fOutput = io.output()
end

function protected:printHeader(run_time)
	self.fOutput:write("\nTime: " .. tostring(run_time) .. "\n")
end

function protected:printErrors(result)
	self:printDefects(result:errors(), result:errorCount(), "error")
end

function protected:printFailures(result)
	self:printDefects(result:failures(), result:failureCount(), "failure")
end

function protected:printDefects(booBoos, count, type)
	if count == 0 then return nil end
	if count == 1
		then self.fOutput:write("There was ", count, " ", type, ":\n")
		else self.fOutput:write("There were ", count, " ", type, "s:\n")
	end
	for i = 1, table.getn(booBoos) do
		self:printDefectHeader(booBoos[i], i)
		self:printDefectTrace(booBoos[i])
	end
end

function protected:printDefectHeader(booBoo, count)
	-- From JUnit: I feel like making this a println, then adding a line giving the
	-- throwable a chance to print something before we get to the stack trace.
	self.fOutput:write(count, ") ", booBoo:failedTest().name, "\n")
end

function protected:printDefectTrace(booBoo)
	self.fOutput:write(tostring(booBoo:errorMessage()), "\n")
end

function protected:printFooter(result)
	if result:wasSuccessful() then
		local plural
		if result:runCount() == 1 then plural = "" else plural = "s" end
		self.fOutput:write("\nOK\n")
		self.fOutput:write(" (", result:runCount(), " test", plural, ")\n")
	else
		self.fOutput:write("\nFAILURES!!!\n")
		self.fOutput:write("\nTests run: " .. result:runCount() ..
		                   ",  Failures: " .. result:failureCount() ..
		                   ",  Errors: " .. result:errorCount() .. "\n" )
	end
	self.fOutput:write("\n");
end

function public:printResult(result, run_time)
	self:printHeader(run_time)
	self:printErrors(result)
	self:printFailures(result)
	self:printFooter(result)
end

function public:printWaitPrompt()
	self.fOutput:write("\n<RETURN> to continue\n")
end

function public:addError(test, messsage)
	self.fOutput:write("E")
end

function public:addFailure(test, messsage)
	self.fOutput:write("F")
end

function public:endTest(test)
end

function public:startTest(test)
	self.fOutput:write(".")
	self.fColumn = self.fColumn + 1
	if self.fColumn >= 40 then
		self.fOutput:write("\n")
		self.fColumn= 0
	end
end
