require "luaunit.framework.TestCase"
require "luaunit.tests.DefaultTestCase"

-------------------------------------------------------------------------------

local tests = {}

function tests:setUp()
	self.result = luaunit.framework.TestResult()
	self.correct_test = luaunit.tests.DefaultTestCase("testOK")
	self.error_test = luaunit.tests.DefaultTestCase("testError")
	self.failure_test = luaunit.tests.DefaultTestCase("testFailure")
	self.test = luaunit.tests.DefaultTestCase()
	function self.test:runTest()
		self.runs = self.runs + 1
	end
end

function tests:tearDown()
	self.result = nil
	self.correct_test = nil
	self.error_test = nil
	self.failure_test = nil
	self.test = nil
end

function tests:CountTestCases(test_name)
	assert(self.test:countTestCases() == 1, "'countTestCases' is not 1 at '" .. test_name .. "'")
end

function tests:TestRun(test_name)
	self.test:run(self.result)
	assert(self.result ~= nil, "'run' not returned a result at '" .. test_name .. "'")
	assert(self.test.setups == 1, "'run' didn't set up the test at '" .. test_name .. "'")
	assert(self.test.teardowns == 1, "'run' didn't tear down the test at '" .. test_name .. "'")
	assert(self.test.runs == 1, "'run' didn't run the test at '" .. test_name .. "'")
	assert(self.result:wasSuccessful(), "'run' generated wrong errors at '" .. test_name .. "'")
end

function tests:CorrectTestRun(test_name)
	local result = self.correct_test:run()
	assert(self.correct_test.setups == 1, "'run' didn't set up the test at '" .. test_name .. "'")
	assert(self.correct_test.teardowns == 1, "'run' didn't tear down the test at '" .. test_name .. "'")
	assert(self.correct_test.runs == 1, "'run' didn't run the test at '" .. test_name .. "'")
	assert(result ~= nil, "'run' not returned a result at '" .. test_name .. "'")
	assert(result:wasSuccessful(), "'run' generated wrong errors at '" .. test_name .. "'")
end

function tests:ErrorTestRun(test_name)
	local result = self.error_test:run()
	assert(self.error_test.setups == 1, "'run' didn't set up the test at '" .. test_name .. "'")
	assert(self.error_test.teardowns == 1, "'run' didn't tear down the test at '" .. test_name .. "'")
	assert(self.error_test.runs == 1, "'run' didn't run the test at '" .. test_name .. "'")
	assert(result ~= nil, "'run' not returned a result at '" .. test_name .. "'")
	assert(result:errorCount() == 1, "'run' generated wrong number of errors at '" .. test_name .. "'")
end

function tests:FailureTestRun(test_name)
	local result = self.failure_test:run()
	assert(self.failure_test.setups == 1, "'run' didn't set up the test at '" .. test_name .. "'")
	assert(self.failure_test.teardowns == 1, "'run' didn't tear down the test at '" .. test_name .. "'")
	assert(self.failure_test.runs == 1, "'run' didn't run the test at '" .. test_name .. "'")
	assert(result ~= nil, "'run' not returned a result at '" .. test_name .. "'")
	assert(result:failureCount() == 1, "'run' generated wrong number of failures at '" .. test_name .. "'")
end

function tests:BareRun(test_name)
	local success = pcall(self.error_test.runBare, self.error_test)
	assert(not success, "'runBare' not raised errors at '" .. test_name .. "'")
	success = pcall(self.failure_test.runBare, self.failure_test)
	assert(not success, "'runBare' not raised failures at '" .. test_name .. "'")
end

return tests