require("luaunit.framework.TestSuite")
require("luaunit.tests.DefaultTestCase")

-------------------------------------------------------------------------------

local tests = {}

function tests:setUp()
	self.result = luaunit.framework.TestResult()
	self.suite = luaunit.framework.TestSuite()
	self.correct_test = luaunit.tests.DefaultTestCase("testOK")
	self.error_test = luaunit.tests.DefaultTestCase("testError")
	self.failure_test = luaunit.tests.DefaultTestCase("testFailure")
end

function tests:tearDown()
	self.result = nil
	self.suite = nil
	self.correct_test = nil
	self.error_test = nil
	self.failure_test = nil
end

function tests:Initialization(test_name)
	assert(self.suite:countTestCases() == 0, "'countTestCases' is not 0 at '" .. test_name .. "'")
	assert(self.suite:testCount() == 0, "'testCount' is not 0 at '" .. test_name .. "'")
	assert(table.getn(self.suite:tests()) == 0, "'tests' is not 0 at '" .. test_name .. "'")
end

function tests:TestAddition(test_name)
	self.suite:addTest(self.correct_test)
	self.suite:addTest(self.error_test)
	self.suite:addTest(self.failure_test)
	assert(self.suite:countTestCases() == 3, "'countTestCases' is not 3 at '" .. test_name .. "'")
	assert(self.suite:testCount() == 3, "'testCount' is not 3 at '" .. test_name .. "'")
	assert(self.suite:testAt(1) == self.correct_test, "'testAt' failed at '" .. test_name .. "'")
	assert(self.suite:testAt(2) == self.error_test, "'testAt' failed at '" .. test_name .. "'")
	assert(self.suite:testAt(3) == self.failure_test, "'testAt' failed at '" .. test_name .. "'")
	local tests = self.suite:tests()
	assert(table.getn(tests) == 3, "'tests' is not 3 at '" .. test_name .. "'")
	assert(tests[1] == self.correct_test, "'tests' failed at '" .. test_name .. "'")
	assert(tests[2] == self.error_test, "'tests' failed at '" .. test_name .. "'")
	assert(tests[3] == self.failure_test, "'tests' failed at '" .. test_name .. "'")
end
-- [[
function tests:SuiteAddition(test_name)
	self.suite:addTestSuite(luaunit.tests.DefaultTestCase)
	assert(self.suite:countTestCases() == 3, "'countTestCases' is not 3 at '" .. test_name .. "'")
	assert(self.suite:testCount() == 1, "'testCount' is not 1 at '" .. test_name .. "'")
	assert(self.suite:testAt(1) ~= nil, "'testAt' failed at '" .. test_name .. "'")
	local tests = self.suite:tests()
	assert(table.getn(tests) == 1, "'tests' is not 1 at '" .. test_name .. "'")
	assert(tests[1] ~= nil, "'tests' failed at '" .. test_name .. "'")
end
--]]
function tests:TestRun(test_name)
	self.suite:runTest(self.error_test, self.result)
	assert(self.error_test.setups == 1, "'run' didn't set up the test at '" .. test_name .. "'")
	assert(self.error_test.teardowns == 1, "'run' didn't tear down the test at '" .. test_name .. "'")
	assert(self.error_test.runs == 1, "'run' didn't run the test at '" .. test_name .. "'")

	assert(self.result:errorCount() == 1, "'errorCount' is not 1 at '" .. test_name .. "' = " .. self.result:errorCount())
	assert(self.result:failureCount() == 0, "'failureCount' is not 0 at '" .. test_name .. "'")
	assert(self.result:runCount() == 1, "'runCount' is not 1 at '" .. test_name .. "'")
	assert(not self.result:shouldStop(), "'shouldStop' is not false at '" .. test_name .. "'")
	assert(not self.result:wasSuccessful(), "'wasSuccessful' is not true at '" .. test_name .. "'")
end

function tests:TestSuiteRun(test_name)
	self.suite:addTest(self.correct_test)
	self.suite:addTest(self.error_test)
	self.suite:addTest(self.failure_test)

	self.suite:run(self.result)

	assert(self.correct_test.setups == 1, "'run' didn't set up the test at '" .. test_name .. "'")
	assert(self.correct_test.teardowns == 1, "'run' didn't tear down the test at '" .. test_name .. "'")
	assert(self.correct_test.runs == 1, "'run' didn't run the test at '" .. test_name .. "'")
	assert(self.error_test.setups == 1, "'run' didn't set up the test at '" .. test_name .. "'")
	assert(self.error_test.teardowns == 1, "'run' didn't tear down the test at '" .. test_name .. "'")
	assert(self.error_test.runs == 1, "'run' didn't run the test at '" .. test_name .. "'")
	assert(self.failure_test.setups == 1, "'run' didn't set up the test at '" .. test_name .. "'")
	assert(self.failure_test.teardowns == 1, "'run' didn't tear down the test at '" .. test_name .. "'")
	assert(self.failure_test.runs == 1, "'run' didn't run the test at '" .. test_name .. "'")

	assert(self.suite:countTestCases() == 3, "'countTestCases' is not 3 at '" .. test_name .. "'")
	assert(self.suite:testCount() == 3, "'testCount' is not 3 at '" .. test_name .. "'")
	assert(self.suite:testAt(1) == self.correct_test, "'testAt' failed at '" .. test_name .. "'")
	assert(self.suite:testAt(2) == self.error_test, "'testAt' failed at '" .. test_name .. "'")
	assert(self.suite:testAt(3) == self.failure_test, "'testAt' failed at '" .. test_name .. "'")
	local tests = self.suite:tests()
	assert(table.getn(tests) == 3, "'tests' is not 3 at '" .. test_name .. "'")
	assert(tests[1] == self.correct_test, "'tests' failed at '" .. test_name .. "'")
	assert(tests[2] == self.error_test, "'tests' failed at '" .. test_name .. "'")
	assert(tests[3] == self.failure_test, "'tests' failed at '" .. test_name .. "'")

	assert(self.result:errorCount() == 1, "'errorCount' is not 1 at '" .. test_name .. "' = " .. self.result:errorCount())
	assert(self.result:failureCount() == 1, "'failureCount' is not 1 at '" .. test_name .. "'")
	assert(self.result:runCount() == 3, "'runCount' is not 3 at '" .. test_name .. "'")
	assert(not self.result:shouldStop(), "'shouldStop' is not false at '" .. test_name .. "'")
	assert(not self.result:wasSuccessful(), "'wasSuccessful' is not true at '" .. test_name .. "'")
end

return tests