local rawset = rawset
local unpack = unpack
local ipairs = ipairs

local table     = require "table"
local coroutine = require "coroutine"
local loop      = require "loop"
local oo        = require "loop.scoped"

module("luaunit.utils.ParameterGenerator",
	loop.define(oo.class({}, require "luaunit.framework.Constructable")),
	loop.seeapi(oo) )
------------------------------------------------------------------------------
local LIST_END = {}
private = {}
protected = {}
public.NIL_VALUE = {}
------------------------------------------------------------------------------
function constructor(self)
	rawset(protected(self), "fValueSet", {})

	local boolean_values = { true, false }
	local number_values = { -1, 0, 1, -0.1, 0.1 }
	local string_values = { "string", "1", [[.,;-+*/?!'"\\]], "\n\t\r" }
	local function_values = { function() end }
	local thread_values = { coroutine.create(function() end) }
	local table_values = {
		{},
		boolean_values,
		number_values,
		string_values,
		function_values,
		thread_values,
	}

	self:defineValueSet("nil", { public.NIL_VALUE })
	self:defineValueSet("boolean", boolean_values)
	self:defineValueSet("number", number_values)
	self:defineValueSet("string", string_values)
	self:defineValueSet("function", function_values)
	self:defineValueSet("thread", thread_values)
	self:defineValueSet("table", table_values)
	--[[
	self:defineValueSet("type_name", {
		"boolean",
		"number",
		"string",
		"function",
		"thread",
		"table"
	})
	local more_tables = { unpack(table_values) }
	for type_pair in self:parameters{"type_name", "type_name"} do
		for value_pair in self:parameters(type_pair) do
			local new_table = { [ value_pair[1] ] = value_pair[2] }
			table.insert(more_tables, new_table)
		end
	end
	self:removeValueSet("type_name")

	self:defineValueSet("table", more_tables)
	--]]
end
------------------------------------------------------------------------------
function private.iteratorFunction(lists, params)
	local pos = table.getn(lists)
	local continue = true
	while continue do
		local list = lists[pos]
		--------------------------------
		if params[pos] == nil then
			params[pos] = public.NIL_VALUE
		end
		--------------------------------
		params[pos] = list[ params[pos] ]
		if params[pos] == LIST_END then
			if pos == 1 then
				return nil
			else
				params[pos] = list[LIST_END]
			end
		else
			continue = false
		end
		---------------------------------------
		if params[pos] == public.NIL_VALUE then
			params[pos] = nil
		end
		---------------------------------------
		pos = pos - 1
	end
	return params
end

function private:createInteratorState(sequence)
	local linkedLists = {}
	local initialState = {}
	for _, set_name in ipairs(sequence) do
		local set = self.fValueSet[set_name]
		local size = table.getn(set)
		if size > 0 then
			local list = {}
			list[LIST_END] = set[1]
			for i = 2, size do
				list[ set[i-1] ] = set[i]
			end
			list[ set[size] ] = LIST_END
			table.insert(linkedLists, list)
			table.insert(initialState, set[size])
		end
	end
	initialState[1] = LIST_END
	return linkedLists, initialState
end
------------------------------------------------------------------------------
function public:defineValueSet(name, values)
	self.fValueSet[name] = values
end

function public:removeValueSet(name)
	self.fValueSet[name] = nil
end

function public:parameters(sequence)
	return
		self.iteratorFunction,
		self:createInteratorState(sequence)
end
------------------------------------------------------------------------------
--[[
local generator = luaunit.utils.ParameterGenerator()
local allTypes = {
	luaunit.utils.ParameterGenerator.public.NIL_VALUE,
	true, false,
	1, 2, 3,
	"string",
	function() end,
	{},
	coroutine.create(function() end),
}
generator:defineValueSet("lua_types", allTypes)
for arrange in generator:parameters{"lua_types", "lua_types"} do
	print(unpack(arrange))
end
--]]