local ipairs = ipairs

local loop      = require "loop"
local oo        = require "loop.scoped"
local oil       = require "oil"
local scheduler = require "scheduler"

local BaseTest = require "oil.tests.bases.BaseTest"

module("oil.tests.bases.MonoTest",
	loop.define(oo.class({}, BaseTest)),
	loop.seeapi(oo) )

oil.loadidl[[
	module OiLTest {
		interface Terminator { void finish(); };
		interface Callback { void notify(); };
	};
]]

function setUp(self)
	BaseTest.setUp(self)
	if self.case.Servers then
		for index, script in ipairs(self.case.Servers) do
			self.system:newLuaTask([[
				local oil = require "oil"
				local Assert = require "luaunit.framework.Assert"
				]]..self.loader:getLoading(self.case)..[[
				]]..script..[[
				oil.loadidl "module OiLTest { interface Terminator { void finish(); }; };"
				OiLTestRegister("OiLTestTerminator_]]..index..[[", oil.newobject(
					{ finish = function() END_OIL = true end },
					"IDL:OiLTest/Terminator:1.0"
				))
				repeat oil.step() until END_OIL
			]])
		end
	end
end

function runTest(self)
	local thread = scheduler.current()
	local callback = oil.newobject(
		{ notify = function() scheduler.wake(thread) end },
		"OiLTest::Callback"
	)
	self.system:register("OiLTestCallback", callback)
	self.system:newLuaTask([[
		local oil = require "oil"
		local Assert = require "luaunit.framework.Assert"
		]]..self.loader:getLoading(self.case)..[[
		]]..self.case.Client..[[
		oil.loadidl "module OiLTest { interface Callback { void notify(); }; };"
		local callback = OiLTestLookUp("OiLTestCallback")
		callback:_narrow("IDL:OiLTest/Callback:1.0")
		callback:notify()
	]])
	scheduler.sleep()
	self.system:unregister("OiLTestCallback")
	callback:_deactivate()
end

function tearDown(self)
	if self.case.Servers then
		for index in ipairs(self.case.Servers) do
			local terminator = self.system:lookup("OiLTestTerminator_"..index, true)
			if terminator then
				terminator:_narrow("IDL:OiLTest/Terminator:1.0")
				terminator:finish()
			end
		end
	end
end
