--
-- Case test by Leonardo Maciel
--

module("oil.tests.cases.NamingService")

NoMonothread = true

IDLFiles = { "CosNaming.idl" }

Services = { "NamingService" }

IDL = [[
	interface Account {
	    void deposit( in unsigned long amount );
	    void withdraw( in unsigned long amount );
	    long balance();
	};
	
	interface Bank {
	    Account create ();
	};
]]

Servers = {
[[
	require "oil.cos.naming"
	OiLTestRegister("OiLNamingService", oil.newobject(oil.cos.naming.new()))
]],
[[
	local Account = oil.oo.class{ bal = 0 }
	function Account:deposit(amount)
		self.bal = self.bal + amount
	end
	function Account:withdraw(amount)
		self.bal = self.bal - amount
	end
	function Account:balance()
		return self.bal
	end
	
	local Bank = {}
	function Bank:create()
		return Account()
	end
	
	local ons = OiLTestLookUp("OiLNamingService")
	ons:_narrow("IDL:omg.org/CosNaming/NamingContextExt:1.0")
	ons:bind(
		{{id="Bank",kind=""}},
		oil.newobject(Bank, "IDL:Bank:1.0")
	)
]],
}

Client = [[
	local ans = OiLTestLookUp("services/NamingService")
	local ons = OiLTestLookUp("OiLNamingService")

	ans:_narrow("IDL:omg.org/CosNaming/NamingContextExt:1.0")
	ons:_narrow("IDL:omg.org/CosNaming/NamingContextExt:1.0")

	local success, bank
	repeat
		success, bank = scheduler.pcall(ons.resolve_str, ons, "Bank")
		if not success then
			Assert:assertEquals(
				bank[1], "IDL:omg.org/CosNaming/NamingContext/NotFound:1.0",
				"invalid raised exception")
		end
	until success
	
	bank = oil.narrow(bank, "IDL:Bank:1.0")
	local account = bank:create()
	account:withdraw(2)
	account:deposit(10)
	Assert:assertEquals(account:balance(), 8, "invalid account balance")

	ans:bind({{id="Bank",kind=""}}, bank)
	
	local cxt = ons:new_context()
	ons:bind_context({{id="ans",kind=""}}, ans) -- ons -> ans, cxt
	ans:bind_context({{id="ons",kind=""}}, ons) -- ans -> ons, cxt
	ons:bind_context({{id="cxt",kind=""}}, cxt) -- cxt -> ons
	ans:bind_context({{id="cxt",kind=""}}, cxt)
	cxt:bind_context({{id="ons",kind=""}}, ons)
	
	Assert:assertNotNil(ons:resolve_str("ans/Bank")            )
	Assert:assertNotNil(ans:resolve_str("ons/Bank")            )
	Assert:assertNotNil(ons:resolve_str("cxt/ons/cxt/ons/Bank"))
	Assert:assertNotNil(ans:resolve_str("ons/ans/Bank")        )

	bank = oil.narrow(ons:resolve_str("ans/ons/cxt/ons/ans/Bank"), "IDL:Bank:1.0")
	account = bank:create()
	account:withdraw(10)
	account:deposit(2)
	Assert:assertEquals(account:balance(), -8, "invalid account balance")
	
	ans:bind({{id="cxt",kind=""},{id="account",kind=""}}, account)
	account = oil.narrow(ons:resolve_str("cxt/account"), "IDL:Account:1.0")
	Assert:assertEquals(account:balance(), -8, "invalid account balance")
	
	ans:unbind({{id="Bank",kind=""}})
	ans:unbind({{id="ons",kind=""}})
	ans:unbind({{id="cxt",kind=""}})
]]
