local assert = assert

local io   = require "io"
local os   = require "os"
local loop = require "loop"
local oo   = require "loop.simple"
local oil  = require "oil"

module("oil.tests.system.SystemWithNS",
	loop.define(oo.class({}, require "oil.tests.system.SystemBase")),
	loop.seeapi(oo) )

LuaCommand = "lua"

local count = 0
function newLuaTask(self, script)
	count = count + 1
	local file = assert(io.open("temp/OiLScript"..count..".lua", "w"))
	file:write([[
		local OiLTestTempFiles = {'temp/OiLScript]]..count..[[.lua'}
		local function OiLTestRegister(name, object)
			name = "temp/"..name..".ior"
			local file = assert(io.open(name, "w"))
			file:write(object:_ior())
			file:close()
			table.insert(OiLTestTempFiles, name)
		end
		local function OiLTestLookUp(name, object, nowait)
			repeat
				local file = io.open("temp/"..name..".ior")
				if file then
					oil.socket.sleep(0.1)
					local ior = file:read("*l")
					file:close()
					return oil.newproxy(ior, "IDL:omg.org/CORBA/Object:1.0")
				end
				oil.socket.sleep(1)
			until nowait
		end
		]], script, [[
		for _, file in ipairs(OiLTestTempFiles) do
			os.remove(file)
		end
	]])
	file:close()
	return self:newTask(
		self:getCommand("Lua", { file_name = "temp/OiLScript"..count..".lua" }),
		"OiL Test Server "..count
	)
end

--------------------------------------------------------------------------------

function register(self, name, object)
	return self.ns:bind({{name=name, kind=""}}, object)
end

function unregister(self, name)
	return self.ns:unbind({{name=name, kind=""}})
end

function lookup(self, name, nowait)
	repeat
		local obj = self.ns:resolve_str(name)
		if obj then
			return obj
		end
		oil.socket.sleep(1)
	until nowait
end

function getIOR(self, name, nowait)
	local obj = self:lookup(name, nowait)
	if obj then
		return oil.idl.encode(obj)
	end
end

--------------------------------------------------------------------------------

function getService(self, service)
	if not self.Services[service] then
		self.Services[service] = self:lookup("services/"..service, true)
		if not self.Services[service] then
			local command = self:getCommand(service, {
				ior_file = "temp/services/"..service..".ior"
			})
			if command then
				self:newTask(command, service)
				self.Services[service] = self:lookup("services/"..service)
			end
		end
	end
	return self.Services[service]
end
