------------------------------------------------------------------------------
-- Project: Serialization of Lua Values                                     --
-- Release: 2.0 alpha                                                       --
-- Title  : Stream of Lua Values for Files                                  --
-- Author : Renato Maia <maia@inf.puc-rio.br>                               --
-- Date   : 12/28/2004 12:00                                                --
------------------------------------------------------------------------------

local assert = assert
local type   = type
local unpack = unpack

local loop          = require "loop"
local oo            = require "loop.simple"
local serialization = require "serialization"

module("serialization.FileStream",
	loop.define(oo.class()),
	loop.seeapi(oo) )

function __init(class, object, mode)
	if type(object) == "userdata" then
		object = { file = object }
	elseif type(object) == "string" then
		object = { file = assert(io.open(object, mode)) }
	elseif type(object) == "table" then
		assert(type(object.file) == "userdata", "invalid file handler")
	end
	return rawnew(class, object)
end

function put(self, ...)
	self.file:write(serialization.serialize(unpack(arg)), "\n")
end

function get(self)
	return serialization.load(self.file:read("*l"))
end

function close(self)
	return self.file:close()
end